<?php

namespace Drupal\gantt\Plugin\views\style;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Drupal\views\Plugin\views\wizard\WizardInterface;

/**
 * Style plugin to render each item in a gantt chart.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "gantt",
 *   title = @Translation("Gantt"),
 *   help = @Translation("Displays rows in a gantt chart."),
 *   theme = "views_view_gantt",
 *   display_types = {"normal"}
 * )
 */
class Gantt extends StylePluginBase {

  /**
   * Does the style plugin for itself support to add fields to its output.
   *
   * @var bool
   */
  protected $usesFields = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = FALSE;

  /**
   * Contains the current active sort column.
   *
   * @var string
   */
  public $active;

  /**
   * Contains the current active sort order, either desc or asc.
   *
   * @var string
   */
  public $order;

  /**
   * Define Options.
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['text'] = ['default' => ''];
    $options['progress'] = ['default' => ''];
    $options['start_date'] = ['default' => ''];
    $options['duration'] = ['default' => ''];
    $options['type'] = ['default' => ''];
    // $options['columns'] = ['default' => ''];
    $options['add_task'] = ['default' => TRUE];
    $options['edit_task'] = ['default' => TRUE];
    $options['open'] = ['default' => ''];
    $options['cdn'] = ['default' => TRUE];
    $options['send_email'] = ['default' => FALSE];
    $options['send_notification'] = ['default' => FALSE];
    $options['links'] = ['default' => ''];
    $options['parent'] = ['default' => ''];
    $options['owner'] = ['default' => ''];
    return $options;
  }

  /**
   * Render the given style.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $optionsNone = ['' => $this->t('- None -')];
    $fields = $this->displayHandler->getHandlers('field');
    $labels = $this->displayHandler->getFieldLabels();
    $field_labels = [];
    foreach ($fields as $field_name => $field) {
      $field_labels[$field_name] = $labels[$field_name];
      if (!empty($field->options["type"])) {
        $type[$field->options["type"]][$field_name] = $labels[$field_name];
      }
      if (!empty($field->multiple)) {
        $multiples[$field_name] = $field->multiple;
      }
    }
    $options = $optionsNone + $field_labels;
    $optionsInt = $optionsUser = $optionsDec = $optionsBigInt = $optionsStatus =
    $optionsDate = $optionsString = $optionsBoolean = $optionsNone;
    if (!empty($type["string"])) {
      $optionsString += $type["string"];
    }
    if (!empty($type["double_field_unformatted_list"])) {
      $optionsString += $type["double_field_unformatted_list"];
    }
    if (!empty($type["text_default"])) {
      $optionsString += $type["text_default"];
    }
    if (!empty($type["number_integer"])) {
      $optionsBigInt += $type["number_integer"];
      $optionsInt += $type["number_integer"];
    }
    if (!empty($type["bigint_item_default"])) {
      $optionsBigInt += $type["bigint_item_default"];
      $optionsInt += $type["bigint_item_default"];
    }
    if (!empty($type["entity_reference_label"])) {
      $optionsStatus += $type["entity_reference_label"];
    }
    if (!empty($type["boolean"])) {
      $optionsBoolean += $type["boolean"];
    }
    if (!empty($type["number_decimal"])) {
      $optionsDec += $type["number_decimal"];
    }
    if (!empty($type["list_default"])) {
      $optionsBigInt += $type["list_default"];
      $optionsInt += $type["list_default"];
    }
    if (!empty($type["daterange_default"])) {
      $optionsDate += $type["daterange_default"];
    }
    if (!empty($type["entity_reference_label"])) {
      $optionsUser += $type["entity_reference_label"];
    }

    $form['parent'] = [
      '#type' => 'select',
      '#title' => $this->t('Parent ID field'),
      '#options' => $optionsInt,
      '#required' => TRUE,
      '#default_value' => !empty($this->options['parent']) ? $this->options['parent'] : '',
      '#description' => $this->t("Select the field that contains nid of the record's parent node."),
    ];
    $form['links'] = [
      '#type' => 'select',
      '#title' => $this->t('Link field'),
      '#options' => $optionsString,
      '#default_value' => !empty($this->options['links']) ? $this->options['links'] : '',
      '#description' => $this->t("Select the field that contains nid and type of the record's predecessor node."),
    ];
    $form['text'] = [
      '#type' => 'select',
      '#title' => $this->t('Name field'),
      '#options' => $optionsString,
      '#default_value' => $this->options['text'],
      '#description' => $this->t('Select the field that contains name of each record.'),
      '#required' => TRUE,
    ];
    $form['start_date'] = [
      '#type' => 'select',
      '#title' => $this->t('Date field'),
      '#options' => $optionsDate,
      '#default_value' => !empty($this->options['start_date']) ? $this->options['start_date'] : '',
      '#description' => $this->t('Select the field that contains the start date (timestamp or valid <a href="http://www.php.net/manual/en/datetime.formats.php" target="_blank">date format</a>) of the node in the selected row.'),
      '#required' => TRUE,
    ];
    $form['duration'] = [
      '#title' => $this->t('Duration'),
      '#description' => $this->t('Duration Field'),
      '#type' => 'select',
      '#empty_option' => $this->t('- None -'),
      '#options' => $optionsInt,
      '#default_value' => !empty($this->options['duration']) ? $this->options['duration'] : '',
    ];
    $form['open'] = [
      '#title' => $this->t('Open'),
      '#description' => $this->t('Field open'),
      '#type' => 'select',
      '#options' => $optionsBoolean,
      '#default_value' => !empty($this->options['open']) ? $this->options['open'] : '',
    ];
    $form['progress'] = [
      '#type' => 'select',
      '#title' => $this->t('Progress field'),
      '#options' => $optionsDec,
      '#default_value' => !empty($this->options['progress']) ? $this->options['progress'] : '',
      '#description' => $this->t('Select the field that contains the progress of the node in percents.'),
    ];
    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Task type'),
      '#options' => $optionsStatus,
      '#default_value' => !empty($this->options['type']) ? $this->options['type'] : '',
      '#description' => $this->t("Select the task node type."),
    ];
    $form['owner'] = [
      '#type' => 'select',
      '#title' => $this->t('Assignor'),
      '#options' => $optionsUser,
      '#default_value' => !empty($this->options['owner']) ? $this->options['owner'] : '',
      '#description' => $this->t("Select the task owner field."),
    ];
    /*
    $form['columns'] = [
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => $this->t('Additional columns'),
    '#options' => $options,
    '#default_value' => !empty($this->options['columns']) ?
    $this->options['columns'] : '',
    ];
     */
    $form['add_task'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add Task'),
      '#default_value' => !empty($this->options['add_task']) ? $this->options['add_task'] : '',
      '#description' => $this->t("Enable posibility 'Add Task' from Gantt chart."),
    ];
    $form['edit_task'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Edit Task'),
      '#default_value' => !empty($this->options['edit_task']) ? $this->options['edit_task'] : '',
      '#description' => $this->t("Enable posibility 'Edit Task' from Gantt chart."),
    ];

    $form['cdn'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use CDN'),
      '#description' => $this->t('If not you can use in /libraries/gantt/codebase/dhtmlxgantt.js'),
      '#default_value' => !empty($this->options['cdn']) ? $this->options['cdn'] : '',
    ];
    $form['send_email'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send an email after the change of status'),
      '#default_value' => !empty($this->options['send_email']) ? $this->options['send_email'] : '',
    ];
    if (\Drupal::moduleHandler()->moduleExists('pwa_firebase')) {
      $form['send_notification'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Send notification to assign user'),
        '#default_value' => !empty($this->options['send_notification']) ? $this->options['send_notification'] : '',
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function wizardSubmit(&$form, FormStateInterface $form_state, WizardInterface $wizard, &$display_options, $display_type) {
    // If any of the displays use the table style, make sure that the fields
    // always have a labels by unsetting the override.
    foreach ($display_options['default']['fields'] as &$field) {
      unset($field['label']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return Cache::PERMANENT;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return [];
  }

}
